#include "/lib/noise.glsl"

struct SkyColors {
    vec3 ground;
    vec3 top;
};

SkyColors calc_sky_coef() {
    SkyColors FinalSky;
    const vec3 SKY_TOP_NOON = vec3(0.2, 0.36, 0.7);
    const vec3 SKY_TOP_SUNRISE = vec3(0.1, 0.25, 0.5);
    const vec3 SKY_TOP_SUNSET = vec3(0.07, 0.12, 0.15);
    const vec3 SKY_TOP_NIGHT = vec3(0.02, 0.03, 0.05);

    const vec3 SKY_GROUND_NOON = vec3(0.6, 0.65, 0.9);
    const vec3 SKY_GROUND_SUNRISE = vec3(1.0, 0.55, 0.75);
    const vec3 SKY_GROUND_SUNSET = vec3(1.0, 0.4, 0.2);
    const vec3 SKY_GROUND_NIGHT = vec3(0.15, 0.2, 0.25);

    if(worldTime > 23500 || worldTime <= 500) {
        float x = sin(PI*(worldTime+500)/2000.0);
        FinalSky.top = mix(SKY_TOP_NIGHT, SKY_TOP_SUNRISE, x);
        FinalSky.ground = mix(SKY_GROUND_NIGHT, SKY_GROUND_SUNRISE, x);
    }
    else if(worldTime <= 1500) {
        float x = sin(PI*(worldTime-500)/2000.0);
        FinalSky.top = mix(SKY_TOP_SUNRISE, SKY_TOP_NOON, x);
        FinalSky.ground = mix(SKY_GROUND_SUNRISE, SKY_GROUND_NOON, x);
    }
    else if(worldTime <= 11500) {
        FinalSky.top = SKY_TOP_NOON;
        FinalSky.ground = SKY_GROUND_NOON;
    }
    else if(worldTime <= 12500) {
        float x = sin(PI*(worldTime+500.0)/2000.0);
        FinalSky.top = mix(SKY_TOP_NOON, SKY_TOP_SUNSET, x);
        FinalSky.ground = mix(SKY_GROUND_NOON, SKY_GROUND_SUNSET, x);
    }
    else if(worldTime <= 13500) {
        float x = sin(PI*(worldTime-500.0)/2000.0);
        FinalSky.top = mix(SKY_TOP_SUNSET, SKY_TOP_NIGHT, x);
        FinalSky.ground = mix(SKY_GROUND_SUNSET, SKY_GROUND_NIGHT, x);
    }
    else if(worldTime <= 23500) {
        FinalSky.top = SKY_TOP_NIGHT;
        FinalSky.ground = SKY_GROUND_NIGHT;
    }
    FinalSky.top = apply_saturation(FinalSky.top, 1 - rainStrength) * (1 - rainStrength*0.25);
    FinalSky.ground = apply_saturation(FinalSky.ground, 1 - rainStrength/2)* (1 - rainStrength*0.33);
    return FinalSky;
} 

float fogify(float x, float w) {
	return w / (x * x + w);
}

vec3 get_clouds(vec3 ViewPos, vec3 CloudPos) {
    #if CLOUD_SPEED != 0
        #ifdef DIMENSION_OVERWORLD
        const int ACTUAL_CLOUD_SPEED = 75-CLOUD_SPEED*10;
        float Animation = frameTimeCounter/ACTUAL_CLOUD_SPEED+float(worldTime)*PI/5000;
        const int ACTUAL_CLOUD_SIZE = CLOUD_SIZE;
        #else
        const int ACTUAL_CLOUD_SPEED = 20-CLOUD_SPEED*2;
        float Animation = frameTimeCounter/ACTUAL_CLOUD_SPEED;
        const int ACTUAL_CLOUD_SIZE = 1;
        #endif
    #else
        float Animation = 0;
    #endif
	float Noise = fbm(CloudPos.xz+Animation, CLOUD_QUALITY);
    Noise = pow(Noise, CLOUD_AMOUNT - rainStrength*2)*ACTUAL_CLOUD_SIZE;

    // Reduce cloud amount further away
    Noise = Noise * (1 - clamp(length(CloudPos.xz/CloudPos.y/24), 0, 1));
    
    #ifndef DIMENSION_END
    Noise = ACESFilm(vec3(Noise)).r; // Yes, I'm tonemapping the clouds. Cry about it.
    SkyColors SkyCol = calc_sky_coef();
	return dither(Noise * SkyCol.ground, CloudPos.xz);
    #else
    return dither(pow(Noise, 1.33) * vec3(40, 15, 2), CloudPos.xz);
    #endif
}

vec3 calcSkyColor(vec3 pos) {
	float upDot = dot(pos, gbufferModelView[1].xyz)+0.1; //not much, what's up with you?
    SkyColors FinalSky = calc_sky_coef();
	return dither(mix(FinalSky.top, FinalSky.ground, fogify(max(upDot, 0.0), 0.05)), pos.xy);
}
