vec4 write_bloom_buffer(vec4 Color) {
    float Luminance = get_luminance(Color.xyz)-0.5;
    #ifndef DIMENSION_NETHER
        Luminance /= 2;
    #endif
    return mix(vec4(0), Color, max(Luminance, 0));
}

// Returns the bloom tile you are currently on
int is_in_range(vec2 texcoord) { 
    vec2 pixels = vec2(viewWidth, viewHeight);
    vec2 accepted_range[2] = vec2[2](vec2(0), vec2(0));
    for(int i = 2; i <= 2+BLOOM_SIZE; i++) {
        accepted_range[0] = accepted_range[1]+vec2(1);
        accepted_range[1] += pixels/pow(2, i);
        if(all(greaterThan(texcoord*pixels, accepted_range[0])) && all(lessThan(texcoord*pixels, accepted_range[1]))) {
            return i;
        }
    }
    return 0;
}

// https://github.com/Experience-Monks/glsl-fast-gaussian-blur
vec4 blur(sampler2D image, vec2 uv, vec2 resolution, vec2 direction) {
    if(is_in_range(uv) == 0) 
        return vec4(0, 0, 0, 1);
    vec4 color = vec4(0.0);
    vec2 off1 = vec2(1.3333333333333333) * direction;
    color += texture2D(image, uv) * 0.29411764705882354;
    color += texture2D(image, uv + off1 / resolution) * 0.35294117647058826;
    color += texture2D(image, uv - off1 / resolution) * 0.35294117647058826;
    return color; 
}


